@@ -17,19 +17,22 @@ use configparser::ini;
 use structopt::StructOpt;
 use wildmatch::WildMatch;
 
-/// `process::Command` to run, including additional information.  The `output` must be set manually
-/// after executing the command found in structs `cmdline`.
+/// The final `process::Command` to execute, bundled together with related information for quick
+/// access.  Those related path informations must be set manually when building the `cmdline`.
+/// They point to the arguments used in the command, such as the path to a `game`.  The `output`
+/// must be set manually after executing the `cmdline` process.
 #[derive(Debug)]
 pub struct RunCommand {
-    //
     pub cmdline: Command,
     pub game: PathBuf,
     pub libretro: PathBuf,
     pub output: Option<Output>,
 }
 
-/// Program settings.  Intended to be filled from various sources, like user configuration or
-/// commandline arguments.  This serves as the source to build the final `retroarch` command.
+/// Configuration of the main program.  The intended use case is to create multiple `Settings` data
+/// from various places like commandline arguments or user configuration file.  Then all those
+/// `Settings` data should be merged into a single one, which will be used as the source when
+/// finally building the `RunCommand`.  Which is then used to execute `retroarch` program itself.
 #[derive(Debug)]
 pub struct Settings {
     games: Vec<PathBuf>,
@@ -107,10 +110,12 @@ impl Settings {
     }
 
     /// Parse own commandline arguments and create a new Settings struct out of it.
-    pub fn new_from_cmdline(options: Option<Vec<String>>) -> Result<Settings, Box<dyn Error>> {
+    pub fn new_from_cmdline(
+        options: Option<Vec<String>>,
+    ) -> Result<Settings, Box<dyn Error>> {
         let mut settings: Settings = Settings::new();
 
-        let args = match options {
+        let args: Opt = match options {
             Some(from) => Opt::from_iter(from.iter()),
             None => Opt::from_args(),
         };
@@ -159,9 +164,11 @@ impl Settings {
         Ok(settings)
     }
 
-    /// Parse the `RetroArch` own configuration `retroarch.cfg` file and create a new Settings struct
-    /// out of it.
-    pub fn new_from_retroarch_config(file: &Option<PathBuf>) -> Result<Settings, Box<dyn Error>> {
+    /// Parse `retroarch.cfg` the own configuration file of `RetroArch` itself and create a new
+    /// `Settings` struct out of it.
+    pub fn new_from_retroarch_config(
+        file: &Option<PathBuf>,
+    ) -> Result<Settings, Box<dyn Error>> {
         let mut settings: Settings = Settings::new();
 
         // If no file was given, then search at `RetroArch` default locations for the file `retroarch.cfg`.
@@ -170,13 +177,14 @@ impl Settings {
             None => retroarch::search_default_config(),
         };
 
-        // The list of key names to search for in the file and limit its parsed output.
+        // The list of key names to search and extract.  Ignore all other.
         let mut keys_to_get: HashSet<String> = HashSet::new();
         keys_to_get.insert("libretro_directory".to_string());
 
-        // Parse.
-        let retroarch_config_map =
-            retroarch::parse_retroarch_config(&settings.retroarch_config, &keys_to_get)?;
+        let retroarch_config_map = retroarch::parse_retroarch_config(
+            &settings.retroarch_config,
+            &keys_to_get,
+        )?;
 
         // Extract values.
         if let Some(value) = retroarch_config_map.get("libretro_directory") {
@@ -186,7 +194,7 @@ impl Settings {
         Ok(settings)
     }
 
-    /// Parse the programs user configuration ini file and create a new Settings struct out of it.
+    /// Parse programs user configuration INI file and create a new `Settings` struct out of it.
     ///
     /// Example structure:
     ///
@@ -210,10 +218,11 @@ impl Settings {
     /// [.md, .gen]
     /// libretro = genesis_plus_gx
     /// ```
-    pub fn new_from_config(file: &Option<PathBuf>) -> Result<Settings, Box<dyn Error>> {
+    pub fn new_from_config(
+        file: &Option<PathBuf>,
+    ) -> Result<Settings, Box<dyn Error>> {
         let mut settings: Settings = Settings::new();
 
-        // `path` is needed as an extra variable, in case of returning an error message later.
         let path: PathBuf = match file {
             Some(p) => p.clone(),
             None => return Ok(settings),
@@ -242,7 +251,8 @@ impl Settings {
 
         // [cores]
         // snes = snes9x
-        let cores_rules = Settings::read_config_cores_rules(&ini);
+        let cores_rules: HashMap<String, PathBuf> =
+            Settings::read_config_cores_rules(&ini);
         if !cores_rules.is_empty() {
             settings.cores_rules.replace(cores_rules);
         }
@@ -250,16 +260,24 @@ impl Settings {
         // [.smc .sfc]
         // core = snes
         // libretro = snes9x
-        let extension_rules =
-            Settings::read_config_extension_rules(&settings.cores_rules, &ini, &section_names);
+        let extension_rules: HashMap<String, PathBuf> =
+            Settings::read_config_extension_rules(
+                &settings.cores_rules,
+                &ini,
+                &section_names,
+            );
         if !extension_rules.is_empty() {
             settings.extension_rules.replace(extension_rules);
         }
 
         // [/home/user/roms/genesis_wide]
         // core = mdwide
-        let directory_rules =
-            Settings::read_config_directory_rules(&settings.cores_rules, &ini, &section_names);
+        let directory_rules: HashMap<String, PathBuf> =
+            Settings::read_config_directory_rules(
+                &settings.cores_rules,
+                &ini,
+                &section_names,
+            );
         if !directory_rules.is_empty() {
             settings.directory_rules.replace(directory_rules);
         }
@@ -274,8 +292,23 @@ impl Settings {
     /// including a dash inside the argument name (in example `--retroarch-config`) will in the
     /// file use underscore instead (in example `retroarch_config`).
     ///
+    /// Read all key/value pairs from section `[options]` in the INI.  Only existing keys are
+    /// updated on the corresponding `Settings` fields, without affecting the others.  These are
+    /// the same options found in the programs commandlline arguments (use `enjoy --help` to find
+    /// out more).  There are only a couple of differences to it:
+    ///
+    ///     - There is only one `game` entry instead of a list.
+    ///     - Some options about the configuration file itself are not present here, because they
+    ///     are evaluated before loading the INI: `--config`, `--open-config` or `--noconfig`.
+    ///     - Flags in the commandline can be used here with a value of `1` or `true` to indicate
+    ///     they are active.  This is needed, because each key or option in the INI file has a
+    ///     value, which is not the case on the commandline.  An example would be `--norun`, which
+    ///     is then translated into INI key `norun = 1`.
+    ///
+    /// ```ini
     /// [options]
     /// retroarch = /usr/bin/retroarch
+    /// ```
     fn read_config_options(
         settings: &mut Settings,
         ini: &ini::Ini,
@@ -323,11 +356,13 @@ impl Settings {
         Ok(())
     }
 
-    /// Read in the alias mappings for `core` names with their associated `libretro` path in the
-    /// section `[cores]` from ini.
+    /// Extract user defined alias mappings for `core` names and their associated `path` in section
+    /// `[cores]`.
     ///
+    /// ```ini
     /// [cores]
     /// snes = snes9x
+    /// ```
     fn read_config_cores_rules(ini: &ini::Ini) -> HashMap<String, PathBuf> {
         let mut cores_rules: HashMap<String, PathBuf> = HashMap::new();
 
@@ -335,7 +370,9 @@ impl Settings {
             // Get valid entries only and convert to `(String, String)`.
             for (core_alias, libretro_path) in cores
                 .iter()
-                .filter(|(_, v)| !v.as_ref().unwrap_or(&"".to_string()).is_empty())
+                .filter(|(_, v)| {
+                    !v.as_ref().unwrap_or(&"".to_string()).is_empty()
+                })
                 .map(|(k, v)| (k.to_string(), v.as_ref().unwrap()))
             {
                 cores_rules.insert(core_alias, PathBuf::from(libretro_path));
@@ -348,11 +385,13 @@ impl Settings {
     /// Read in all rules for the extensions from ini.  `extension_rules` start with a dot in their
     /// section name like `[.smc .sfc]`.  Multiple extensions can be space separated per rule.  The
     /// leading dot will be removed.  Any `core` rule will be resolved to a `libretro` path by
-    /// looking up corresponding alias in `cores_rules`.  But an existing `libretro` rule have
-    /// higher priority.
+    /// looking up corresponding alias in `cores_rules`.  An existing `libretro` rule have higher
+    /// priority over `core` rule.
     ///
+    /// ```ini
     /// [.smc .sfc]
     /// core = snes
+    /// ```
     fn read_config_extension_rules(
         cores_rules: &Option<HashMap<String, PathBuf>>,
         ini: &ini::Ini,
@@ -360,7 +399,9 @@ impl Settings {
     ) -> HashMap<String, PathBuf> {
         let mut extension_rules: HashMap<String, PathBuf> = HashMap::new();
 
-        for pattern_group in section_names.iter().filter(|e| e.starts_with('.')) {
+        for pattern_group in
+            section_names.iter().filter(|e| e.starts_with('.'))
+        {
             // [.smc .sfc]
             // Iterate over each extension and remove their leading dot.
             for ext_pattern in pattern_group
@@ -374,11 +415,16 @@ impl Settings {
                 }
                 // core = snes
                 // Lookup matching libretro path from rules.
-                else if let Some(core_alias) = ini.get(&pattern_group, "core") {
+                else if let Some(core_alias) =
+                    ini.get(&pattern_group, "core")
+                {
                     // [cores]
                     // snes = snes9x
-                    if let Some(path) = cores_rules.as_ref().unwrap().get(&core_alias) {
-                        extension_rules.insert(ext_pattern, PathBuf::from(path));
+                    if let Some(path) =
+                        cores_rules.as_ref().unwrap().get(&core_alias)
+                    {
+                        extension_rules
+                            .insert(ext_pattern, PathBuf::from(path));
                     }
                 }
             }
@@ -387,13 +433,16 @@ impl Settings {
         extension_rules
     }
 
-    /// Read in all rules for the directories from ini.  `directory_rules` include a slash in their
-    /// section name like `[/emulators/roms/psx]`.  The tilde will be expanded to users home
-    /// directory.  Any `core` rule will be resolved to a `libretro` path by looking up
-    /// corresponding alias in `cores_rules`.  But an existing `libretro` rule have higher priority.
+    /// Read in all rules for the directories from ini.  `directory_rules` include a slash
+    /// somewhere in their section name like `[/emulators/roms/psx]`.  The starting tilde will be
+    /// expanded to users home directory.  Any `core` rule will be resolved to a `libretro` path by
+    /// looking up corresponding alias in `cores_rules`.  An existing `libretro` rule have higher
+    /// priority over `core` rule.
     ///
+    /// ```ini
     /// [/home/user/roms/genesis_wide]
     /// core = mdwide
+    /// ```
     fn read_config_directory_rules(
         cores_rules: &Option<HashMap<String, PathBuf>>,
         ini: &ini::Ini,
@@ -423,7 +472,9 @@ impl Settings {
             else if let Some(core_alias) = ini.get(&original, "core") {
                 // [cores]
                 // snes = snes9x
-                if let Some(path) = cores_rules.as_ref().unwrap().get(&core_alias) {
+                if let Some(path) =
+                    cores_rules.as_ref().unwrap().get(&core_alias)
+                {
                     directory_rules.insert(expanded, PathBuf::from(path));
                 }
             }
@@ -432,10 +483,13 @@ impl Settings {
         directory_rules
     }
 
-    /// Merge another `Settings` into current one.  Ignore new keys without `Some` values and
-    /// replace the old ones otherwise.  The `games` key is different, as it will always add
-    /// entries from it to the front of the existing old list.
-    pub fn update_from(&mut self, overwrite: Settings) -> Result<(), Box<dyn Error>> {
+    /// Merge current `Settings` with a new one.  Overwrite values only, if the new value is
+    /// `Some`. The `games` key is different, as the new list in `games` will be prepended to
+    /// current existing list.
+    pub fn update_from(
+        &mut self,
+        overwrite: Settings,
+    ) -> Result<(), Box<dyn Error>> {
         if !overwrite.games.is_empty() {
             if self.games.is_empty() {
                 self.games = overwrite.games;
@@ -506,9 +560,11 @@ impl Settings {
 
     /// Update current Settings from new Settings.  Replace the content only, if the old value is
     /// `None`.  Only a few keys are affected, currently `retroarch`, `retroarch_config`,
-    /// `libretro` and `libretro_directory`.  This method serves as default values to be set at
-    /// last moment, if the user did not already fill in those keys.
-    pub fn update_defaults_from(&mut self, overwrite: Settings) -> Result<(), Box<dyn Error>> {
+    /// `libretro` and `libretro_directory`.
+    pub fn update_defaults_from(
+        &mut self,
+        overwrite: Settings,
+    ) -> Result<(), Box<dyn Error>> {
         if self.retroarch.is_none() {
             self.retroarch = overwrite.retroarch;
         }
@@ -526,12 +582,13 @@ impl Settings {
     }
 
     /// Build up the final `RetroArch` run command from the current Settings.  This is the command
-    /// and its options that is used when running `retroarch` commandline application.  It will be
-    /// wrapped up in a separate `RunCommand` struct, which itself includes the commandline to
+    /// and its options that is used when executing `retroarch` commandline application.  It will
+    /// be wrapped up in a separate `RunCommand` struct, which itself includes the commandline to
     /// execute and a few more data.
     pub fn build_command(&self) -> Result<RunCommand, String> {
         // `--retroarch`
-        let mut command = Command::new(&file::to_str(self.retroarch.as_ref()));
+        let mut command: Command =
+            Command::new(&file::to_str(self.retroarch.as_ref()));
 
         // `game`
         let game: Option<PathBuf> = match self.select_game() {
@@ -553,7 +610,9 @@ impl Settings {
             if let Some(core) = &self.core {
                 match &self.cores_rules {
                     Some(rules) => libretro = rules.get(core).cloned(),
-                    None => return Err("No core rules found in `[cores]`.".into()),
+                    None => {
+                        return Err("No core rules found in `[cores]`.".into())
+                    }
                 };
             }
 
@@ -656,14 +715,15 @@ impl Settings {
         None
     }
 
-    /// Extract the first game entry from current settings `games` list.  If any filter is
+    /// Extract the first game entry from current Settings `games` list.  If any filter is
     /// available, then apply it before extraction.  The comparison is always in lowercase.
     /// Supported special characters are only the star "*", for matching anything and questionmark
     /// "?", for matching a single character.  The filter will be enclosed by stars automatically.
     fn select_game(&self) -> Option<PathBuf> {
         match &self.filter {
             Some(filter) => {
-                let pattern = WildMatch::new(&format!("*{}*", filter.to_lowercase()));
+                let pattern: WildMatch =
+                    WildMatch::new(&format!("*{}*", filter.to_lowercase()));
 
                 for game in &self.games {
                     if pattern.matches(
@@ -700,7 +760,7 @@ impl Settings {
         Ok(false)
     }
 
-    /// Get the user configuration ini file path from `config` option in current Settings.  Default
+    /// Get the user configuration INI file path from `config` option in current Settings.  Default
     /// to `None`, if option `noconfig` is active.
     #[must_use]
     pub fn get_config(&self) -> &Option<PathBuf> {
@@ -751,7 +811,8 @@ impl Settings {
     /// `highlander` is active.  Otherwise its always `false`.
     #[must_use]
     pub fn there_can_only_be_one(&self) -> bool {
-        self.highlander.unwrap_or(false) && retroarch::is_running("retroarch", true)
+        self.highlander.unwrap_or(false)
+            && retroarch::is_running("retroarch", true)
     }
 
     /// Execute the given `Command` to run the program with its arguments and return its `output`.
@@ -760,7 +821,8 @@ impl Settings {
         if self.norun.unwrap_or(false) {
             None
         } else {
-            let output = command.output().expect("Error! Could not run RetroArch.");
+            let output: Output =
+                command.output().expect("Error! Could not run RetroArch.");
             if output.status.to_string() != *"exit code: 0" {
                 eprintln!("Could not run RetroArch. {}", output.status)
             }
@@ -842,7 +904,8 @@ mod tests {
     }
 
     #[test]
-    fn new_from_cmdline_emptygame_then_retroarch() -> Result<(), Box<dyn Error>> {
+    fn new_from_cmdline_emptygame_then_retroarch() -> Result<(), Box<dyn Error>>
+    {
         let mut options: Vec<String> = vec![];
         options.push("enjoy".to_string());
         options.push("".to_string());
@@ -926,7 +989,10 @@ mod tests {
             "md".to_string(),
             PathBuf::from("genesis_plus_gx_libretro.so"),
         );
-        cores_rules.insert("mdwide".to_string(), PathBuf::from("genesis_plus_gx_wide"));
+        cores_rules.insert(
+            "mdwide".to_string(),
+            PathBuf::from("genesis_plus_gx_wide"),
+        );
 
         cores_rules
     }
@@ -936,7 +1002,11 @@ mod tests {
         let mut settings = super::Settings::new();
         let ini = test_ini_template();
 
-        super::Settings::read_config_options(&mut settings, &ini, &["options".to_string()])?;
+        super::Settings::read_config_options(
+            &mut settings,
+            &ini,
+            &["options".to_string()],
+        )?;
 
         assert_eq!(
             Some(PathBuf::from("/usr/bin/retroarch")),
@@ -953,7 +1023,11 @@ mod tests {
         let mut settings = super::Settings::new();
         let ini = test_ini_template();
 
-        super::Settings::read_config_options(&mut settings, &ini, &["options".to_string()])?;
+        super::Settings::read_config_options(
+            &mut settings,
+            &ini,
+            &["options".to_string()],
+        )?;
 
         assert_eq!(Some(false), settings.which);
         assert_eq!(Some(true), settings.norun);
@@ -1051,10 +1125,11 @@ mod tests {
     #[test]
     fn select_game_first() {
         //let ini = test_ini_template();
-        let games: Vec<PathBuf> = ["zelda.smc", "mario.smc", "sonic.md", "game4.gb"]
-            .iter()
-            .map(|g| PathBuf::from(g))
-            .collect();
+        let games: Vec<PathBuf> =
+            ["zelda.smc", "mario.smc", "sonic.md", "game4.gb"]
+                .iter()
+                .map(|g| PathBuf::from(g))
+                .collect();
         let mut settings = super::Settings {
             games: games,
             config: None,
